==> snapconnect_futures-4.0.1a3-py2-none-any.whl: snapconnect_futures/upgrade_manager.py <==
"""Manage firmware and spy file upgrades with asynchronous Futures.

Currently, only 1 script upgrade or firmware upgrade is supported at a time.
"""
import logging
import binascii

from datetime import timedelta

from tornado.gen import coroutine, Future, Return, with_timeout

from snapconnect import snap
from snaplib import ScriptsManager
from snaplib import SnappyUploader


log = logging.getLogger('snapconnect_futures.upgrade')


class FutureUpgradeManager(object):
    def __init__(self, snapconnect):
        """Wrap a SNAP Connect instance with the ability to perform OTA script and firmware uploads with Futures

        :param snap.Snap snapconnect: SNAP Connect instance to wrap.
        """
        self._snap = snapconnect
        self._in_progress = False

        self._snap.set_hook(snap.hooks.HOOK_OTA_UPGRADE_COMPLETE, self._upgrade_complete_hook)
        self._snap.set_hook(snap.hooks.HOOK_OTA_UPGRADE_STATUS, self._upgrade_status_hook)

        self._fw_upgrades_in_progress = {}  # Maps from SNAP address to firmware upgrade future
        self._script_upgrades_in_progress = {}  # Maps from SNAP address to script ugprade future

    def _on_tell_vm_stat(self, arg, val):
        """Intercept tellVmStat calls and pass them to the upload manager."""
        self._snap.spy_upload_mgr.onTellVmStat(self._snap.rpc_source_addr(), arg, val)

    def _on_su_recvd_reboot(self, dummy):
        """Intercept su_recvd_reboot calls and pass them to the upload manager."""
        self._snap.spy_upload_mgr.on_recvd_reboot(self._snap.rpc_source_addr())

    def _upgrade_complete_hook(self, complete_addr, status, message):
        """Hook for firmware upgrade completion."""
        f = self._fw_upgrades_in_progress.pop(binascii.hexlify(complete_addr), None)

        if f:
            if status == snap.OTA_PROGRESS_COMPLETE:
                log.info("Firmware push to %s successful.", complete_addr)
                f.set_result(True)
            else:
                log.error("Firmware push to %s failed: %s (Error Code %s).", complete_addr, message, status)
                f.set_result(False)
        else:
            log.error("Received unexpected firmware complete hook for %s: %s", complete_addr, message)

    def _upgrade_status_hook(self, status_addr, percent_complete):
        """Hook for firmware upgrade status."""
        pass  # pragma: no cover

    def _script_upload_complete_callback(self, snappy_upload_obj, result):
        f = self._script_upgrades_in_progress.pop(binascii.hexlify(snappy_upload_obj.netAddr), None)

        if f:
            if result == SnappyUploader.SNAPPY_PROGRESS_COMPLETE:
                log.info("Script push to %s successful.", snappy_upload_obj.netAddr)
                f.set_result(True)
            else:
                log.error("Script push to %s failed (Error Code %r).", snappy_upload_obj.netAddr, result)
                f.set_result(False)
        else:
            log.error("Received unexpected script complete hook for %s", snappy_upload_obj.netAddr)

    @coroutine
    def upgrade_script(self, addr, spy_file_path, timeout):
        """Return a future that resolves to the success when a script upgrade completes.

        Raises a TimeoutError if we don't resolve in time

        :param str addr: SNAP Address (Ex: "aabbcc")
        :param str spy_file_path: Full path to SNAPpy SPY file
        :param float timeout: Time in seconds before an upgrade has failed
        """
        if self._in_progress:
            log.error("Aborting script upgrade for %s.  Upgrade already in progress.", addr)
            raise Return(False)

        log.info("Pushing script %s to %s", spy_file_path, addr)
        self._in_progress = True

        self._snap.local_dispatcher.funcs["tellVmStat"] = self._on_tell_vm_stat
        self._snap.local_dispatcher.funcs["su_recvd_reboot"] = self._on_su_recvd_reboot

        try:
            with open(spy_file_path, "rb") as spy_file:
                upload_image = ScriptsManager.getSnappyStringFromExport(spy_file.read())

            f = Future()
            self._script_upgrades_in_progress[addr] = f

            upload = self._snap.spy_upload_mgr.startUpload(binascii.unhexlify(addr), upload_image)
            upload.registerFinishedCallback(self._script_upload_complete_callback)

            result = yield with_timeout(timedelta(seconds=timeout), f)

            raise Return(result)
        finally:
            # Remove callback functions
            self._snap.local_dispatcher.funcs.pop("tellVmStat", None)
            self._snap.local_dispatcher.funcs.pop("su_recvd_reboot", None)
            self._script_upgrades_in_progress.pop(addr, None)  # Clear the address when we're done
            self._in_progress = False

    @coroutine
    def upgrade_firmware(self, addr, sfi_file_path, timeout):
        """Return a future that resolves to the success when a firmware upgrade completes.

        Raises a TimeoutError if we don't resolve in time

        :param str addr: SNAP Address (Ex: "aabbcc")
        :param str sfi_file_path: Full path to SNAP Firmware Image file
        :param float timeout: Time in seconds before an upgrade has failed
        """
        if self._in_progress:
            log.error("Aborting firmware upgrade for %s.  Upgrade already in progress.", addr)
            raise Return(False)

        log.info("Pushing firmware %s to %s", sfi_file_path, addr)
        self._in_progress = True
        try:
            f = Future()

            self._fw_upgrades_in_progress[addr] = f

            upgrade_started = self._snap.upgrade_firmware(binascii.unhexlify(addr), sfi_file_path)

            if upgrade_started:
                result = yield with_timeout(timedelta(seconds=timeout), f)
            else:
                log.error("Firmware push to %s failed: Unable to start upgrade.", addr)
                result = False

            raise Return(result)
        finally:
            self._fw_upgrades_in_progress.pop(addr, None)  # Clear the address when we're done
            self._in_progress = False

==> snapconnect_futures-5.0.1-py2-none-any.whl: snapconnect_futures/upgrade_manager.py <==
"""Manage firmware and spy file upgrades with asynchronous Futures.

Currently, only 1 script upgrade or firmware upgrade is supported at a time.
"""
import logging
import binascii

from datetime import timedelta

from tornado.gen import coroutine, Future, Return, TimeoutError, with_timeout

from snapconnect import snap
from snaplib import ScriptsManager
from snaplib import SnappyUploader


log = logging.getLogger('snapconnect_futures.upgrade')


class FutureUpgradeManager(object):
    def __init__(self, snapconnect):
        """Wrap a SNAP Connect instance with the ability to perform OTA script and firmware uploads with Futures

        :param snap.Snap snapconnect: SNAP Connect instance to wrap.
        """
        self._snap = snapconnect
        self._in_progress = False

        self._snap.set_hook(snap.hooks.HOOK_OTA_UPGRADE_COMPLETE, self._upgrade_complete_hook)
        self._snap.set_hook(snap.hooks.HOOK_OTA_UPGRADE_STATUS, self._upgrade_status_hook)

        self._fw_upgrades_in_progress = {}  # Maps from SNAP address to firmware upgrade future
        self._script_upgrades_in_progress = {}  # Maps from SNAP address to script ugprade future

    def _on_tell_vm_stat(self, arg, val):
        """Intercept tellVmStat calls and pass them to the upload manager."""
        self._snap.spy_upload_mgr.onTellVmStat(self._snap.rpc_source_addr(), arg, val)

    def _on_su_recvd_reboot(self, dummy):
        """Intercept su_recvd_reboot calls and pass them to the upload manager."""
        self._snap.spy_upload_mgr.on_recvd_reboot(self._snap.rpc_source_addr())

    def _upgrade_complete_hook(self, complete_addr, status, message):
        """Hook for firmware upgrade completion."""
        f = self._fw_upgrades_in_progress.pop(binascii.hexlify(complete_addr), None)

        if f:
            if status == snap.OTA_PROGRESS_COMPLETE:
                log.info("Firmware push to %s successful.", complete_addr)
                f.set_result(True)
            else:
                log.error("Firmware push to %s failed: %s (Error Code %s).", complete_addr, message, status)
                f.set_result(False)
        else:
            log.error("Received unexpected firmware complete hook for %s: %s", complete_addr, message)

    def _upgrade_status_hook(self, status_addr, percent_complete):
        """Hook for firmware upgrade status."""
        pass  # pragma: no cover

    def _script_upload_complete_callback(self, snappy_upload_obj, result):
        f = self._script_upgrades_in_progress.pop(binascii.hexlify(snappy_upload_obj.netAddr), None)

        if f:
            if result == SnappyUploader.SNAPPY_PROGRESS_COMPLETE:
                log.info("Script push to %s successful.", snappy_upload_obj.netAddr)
                f.set_result(True)
            else:
                log.error("Script push to %s failed (Error Code %r).", snappy_upload_obj.netAddr, result)
                f.set_result(False)
        else:
            log.error("Received unexpected script complete hook for %s", snappy_upload_obj.netAddr)

    @coroutine
    def upgrade_script(self, addr, spy_file_path, timeout):
        """Return a future that resolves to the success when a script upgrade completes.

        Raises a TimeoutError if we don't resolve in time

        :param str addr: SNAP Address (Ex: "aabbcc")
        :param str spy_file_path: Full path to SNAPpy SPY file
        :param float timeout: Time in seconds before an upgrade has failed
        """
        if self._in_progress:
            log.error("Aborting script upgrade for %s.  Upgrade already in progress.", addr)
            raise Return(False)

        log.info("Pushing script %s to %s", spy_file_path, addr)
        self._in_progress = True

        self._snap.local_dispatcher.funcs["tellVmStat"] = self._on_tell_vm_stat
        self._snap.local_dispatcher.funcs["su_recvd_reboot"] = self._on_su_recvd_reboot

        try:
            with open(spy_file_path, "rb") as spy_file:
                upload_image = ScriptsManager.getSnappyStringFromExport(spy_file.read())

            f = Future()
            self._script_upgrades_in_progress[addr] = f

            upload = self._snap.spy_upload_mgr.startUpload(binascii.unhexlify(addr), upload_image)
            upload.registerFinishedCallback(self._script_upload_complete_callback)

            try:
                result = yield with_timeout(timedelta(seconds=timeout), f)
            except TimeoutError:
                # Cancel the upgrade if it timed out
                upload.cancel()
                raise

            raise Return(result)
        finally:
            # Remove callback functions
            self._snap.local_dispatcher.funcs.pop("tellVmStat", None)
            self._snap.local_dispatcher.funcs.pop("su_recvd_reboot", None)
            self._script_upgrades_in_progress.pop(addr, None)  # Clear the address when we're done
            self._in_progress = False

    @coroutine
    def upgrade_firmware(self, addr, sfi_file_path, timeout):
        """Return a future that resolves to the success when a firmware upgrade completes.

        Raises a TimeoutError if we don't resolve in time

        :param str addr: SNAP Address (Ex: "aabbcc")
        :param str sfi_file_path: Full path to SNAP Firmware Image file
        :param float timeout: Time in seconds before an upgrade has failed
        """
        if self._in_progress:
            log.error("Aborting firmware upgrade for %s.  Upgrade already in progress.", addr)
            raise Return(False)

        log.info("Pushing firmware %s to %s", sfi_file_path, addr)
        self._in_progress = True
        try:
            f = Future()

            self._fw_upgrades_in_progress[addr] = f

            unhexlified_addr = binascii.unhexlify(addr)
            upgrade_started = self._snap.upgrade_firmware(unhexlified_addr, sfi_file_path)

            if upgrade_started:
                try:
                    result = yield with_timeout(timedelta(seconds=timeout), f)
                except TimeoutError:
                    # Cancel the upgrade if it timed out

                    # This hack looks up the firmware upgrade instance because upgrade_firmware only
                    # returns an "upgrade started" boolean and not the firmware upgrade instance
                    upgrade = snap._upgrades.get(unhexlified_addr)
                    if upgrade:
                        upgrade.cancel()
                    raise
            else:
                log.error("Firmware push to %s failed: Unable to start upgrade.", addr)
                result = False

            raise Return(result)
        finally:
            self._fw_upgrades_in_progress.pop(addr, None)  # Clear the address when we're done
            self._in_progress = False
